package cz.muni.fi.pb162.hw03.csv.impl;

import cz.muni.fi.pb162.hw03.csv.CsvReader;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static cz.muni.fi.pb162.hw03.csv.Messages.INVALID_FORMAT;

public class ReaderWithHeader implements CsvReader<Map<String, String>> {
    private final Reader reader;
    private final List<String> labels;

    public ReaderWithHeader(InputStream is, Charset charset, String delimiter) throws IOException {
        reader = new Reader(is,charset,delimiter);
        labels = reader.read();
    }

    @Override
    public Map<String, String> read() throws IOException {
        List<String> s = reader.read();
        if (s == null){
            return null;
        }
        if (s.size() != labels.size()){
            throw new IOException(INVALID_FORMAT);
        }
        return IntStream.range(0, labels.size()).boxed()
                .collect(Collectors.toMap(labels::get, s::get));
    }

    @Override
    public void forEach(Consumer<Map<String, String>> consumer) throws IOException {
        Map<String, String> line;
        while ((line = read()) != null){
            consumer.accept(line);
        }
    }

    @Override
    public void close() throws IOException {
        reader.close();
    }
}
